package accesspoint;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
public class AccessPoint {
    private int zone;
    private BluetoothAccess ba;
    private int newVisit;
    private int fileServerPort;
    public AccessPoint (String configurationFile) {
        this.configureAccessPoint(configurationFile);
        System.out.println("Zone: "+this.zone);
        System.out.println("New visit: "+this.newVisit);
        System.out.println("File server's port: "+this.fileServerPort);
        System.out.println();
    }
    public void start() {
        this.ba = new BluetoothAccess(this.zone, this.newVisit, this.fileServerPort);
        this.ba.start();
    }
    private void configureAccessPoint (String ConfigurationFile) {
        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = null;
        Document doc = null;
        try {
            docBuilder = docBuilderFactory.newDocumentBuilder();
        }
        catch (ParserConfigurationException e) {
            System.out.println("ERROR - Imposible crear un nuevo DocumentBuilder: "+e);
            return;
        }
        try {
            doc = docBuilder.parse (new File(ConfigurationFile));
        }
        catch (IOException e) {
            System.out.println("ERROR - Problemas de lectura del fichero de configuracion: "+e);
            return;
        }
        catch (SAXException e) {
            System.out.println("ERROR - Problemas de transformacion del fichero de configuracion: "+e);
            return;
        }
        catch (IllegalArgumentException e) {
            System.out.println("ERROR - Fichero de configuracion no localizado: "+e);
            return;
        }
        doc.getDocumentElement().normalize();
        NodeList portList = doc.getElementsByTagName("zone");
        if (portList.getLength() == 1) {
            Element portElement = (Element)portList.item(0);
            NodeList portNumber = portElement.getChildNodes();
            this.zone = Integer.parseInt(((Node)portNumber.item(0)).getNodeValue().trim());
        }
        else {
            System.out.println("ERROR - No existe una unica zona en la configuracion");
            return;
        }
        NodeList newVisitList = doc.getElementsByTagName("newVisit");
        if (newVisitList.getLength() == 1) {
            Element newVisitElement = (Element)newVisitList.item(0);
            NodeList newVisitNumber = newVisitElement.getChildNodes();
            this.newVisit = Integer.parseInt(((Node)newVisitNumber.item(0)).getNodeValue().trim());
        }
        else {
            System.out.println("ERROR - No existe un unico periodo de nueva visita en la configuracion");
            return;
        }
        NodeList fileServerPortList = doc.getElementsByTagName("fileServerPort");
        if (fileServerPortList.getLength() == 1) {
            Element fileServerPortElement = (Element)fileServerPortList.item(0);
            NodeList fileServerPortNumber = fileServerPortElement.getChildNodes();
            this.fileServerPort = Integer.parseInt(((Node)fileServerPortNumber.item(0)).getNodeValue().trim());
        }
        else {
            System.out.println("ERROR - No existe un unico puerto al servidor de ficheros en la configuracion");
            return;
        }
    }
}