package accesspoint;
import java.util.Hashtable;
import java.util.LinkedList;
import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.UUID;
import java.util.Date;
import java.io.BufferedWriter;
import java.io.FileWriter;
import database.DatabaseAccess;
public class BluetoothAccess {
    private BluetoothListener bluetoothListener;
    private DatabaseAccess database;
    private LinkedList<RemoteDevice> devices;
    private DiscoveryAgent discoveryAgent;
    private FileServerAccess fileServer;
    private LocalDevice localDevice;
    private int maxServiceSearch;
    private Hashtable <Integer,RemoteDevice> serviceDevices;
    private UUID[] uuid;
    private int zone;
    private long newVisit;
    
    private int inquiryCount = 0;
    
    public BluetoothAccess (int zone, long newVisit, int fileServerPort) {
        this.database = new DatabaseAccess();
        this.devices = new LinkedList<RemoteDevice>();
        this.fileServer = new FileServerAccess("localhost", fileServerPort);
        this.serviceDevices = new Hashtable <Integer, RemoteDevice> ();
        this.uuid = new UUID[] { new UUID(0x1105) };
        this.zone = zone;
        this.newVisit = newVisit;
        try {
            this.localDevice = LocalDevice.getLocalDevice();
        }
        catch (BluetoothStateException e) {
            System.out.println("ERROR - El sistema Bluetooth no puede ser inicializado: "+e);
            return;
        }
        this.maxServiceSearch = Integer.parseInt(LocalDevice.getProperty("bluetooth.sd.trans.max"));
        this.bluetoothListener = new BluetoothListener(maxServiceSearch, this.database, this.fileServer, this.zone, this.newVisit);
        this.bluetoothListener.setDevices(this.devices);
        this.bluetoothListener.setServiceDevices(this.serviceDevices);
        this.discoveryAgent = this.localDevice.getDiscoveryAgent();
        System.out.println("MaxServiceSearch: "+maxServiceSearch);
    }
    public void deleteOldEntries () {
        this.database.deleteRecentDispositives(this.zone, this.newVisit);
    }
    public void handleDevices () {
        if (this.devices.size() != 0) {
            while ((this.bluetoothListener.getCurrentServiceSearch() < this.maxServiceSearch) && (!this.devices.isEmpty())) {
                this.bluetoothListener.addServiceSearch();
                RemoteDevice remoteDevice = this.devices.removeFirst();
                this.serviceDevices.put(this.searchServices(remoteDevice), remoteDevice);
                System.out.println("Buscando servicios : "+remoteDevice.getBluetoothAddress());
            }
            try {
                synchronized (this.bluetoothListener) {
                    this.bluetoothListener.wait();
                }
            }
            catch (InterruptedException e) {
                System.out.println("WARNING - Busqueda de servicios interrumpida");
            }
        }
    }
    public void searchDevices () {
        System.out.println();
        System.out.println("-------------------------------------------------");
        System.out.println();
        System.out.println("Nuevo inquiry: "+(++inquiryCount));
        try {
            if (!this.discoveryAgent.startInquiry(DiscoveryAgent.GIAC, this.bluetoothListener)) {
                System.out.println("ERROR - Inquiry no iniciado, codigo de acceso GIAC no soportado");
                return;
            }
            else {
                Date date = new Date();
                this.bluetoothListener.startInquiry = date.getTime();
                try {
                    synchronized (this.bluetoothListener) {
                        this.bluetoothListener.wait();
                    }
                }
                catch (InterruptedException e) {
                    System.out.println("WARNING - Busqueda de dispositivos interrumpida");
                }
            }
        }
        catch (BluetoothStateException e) {
            System.out.println("ERROR - El dispositivo Bluetooth esta ocupado y no puede iniciar el inquiry: "+e);
            return;
        }
    }
    public int searchServices(RemoteDevice remoteDevice) {
        try {
            return this.discoveryAgent.searchServices(null, this.uuid, remoteDevice, this.bluetoothListener);
        }
        catch (BluetoothStateException e) {
            System.out.println("ERROR - Imposible iniciar una nueva busqueda de servicios: "+e);
            return 0;
        }
    }
    public void start () {
        while(true) {
            //this.deleteOldEntries();
            this.searchDevices();
            if (this.bluetoothListener.firstTime) this.bluetoothListener.firstTime = false;
            this.handleDevices();
            
        }
    }
}