package accesspoint;
import java.io.IOException;
import java.util.Hashtable;
import java.util.LinkedList;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import java.util.Date;
import java.io.BufferedWriter;
import java.io.FileWriter;
import database.DatabaseAccess;

public class BluetoothListener implements DiscoveryListener {
    private int currentServiceSearch;
    private DatabaseAccess database;
    private LinkedList<RemoteDevice> devices;
    private FileServerAccess fileServer;
    private Hashtable<Integer, RemoteDevice> serviceDevices;
    private int zone;
    private long newVisit;
    
    public int inquiryDeviceNumber = 0;
    public boolean firstTime = true;
    public long startInquiry = 0;
    public int serveCount = 0;
    public LinkedList<RemoteDevice> servir = new LinkedList<RemoteDevice>();
    public LinkedList<RemoteDevice> sirviendo = new LinkedList<RemoteDevice>();
    
    public BluetoothListener (int maxServiceSearch, DatabaseAccess database, FileServerAccess fileServer, int zone, long newVisit) {
        this.currentServiceSearch = 0;
        this.database = database;
        this.devices = new LinkedList<RemoteDevice>();
        this.fileServer = fileServer;
        this.serviceDevices = new Hashtable<Integer, RemoteDevice> ();
        this.zone = zone;
        this.newVisit = newVisit;
    }
    public int getCurrentServiceSearch () {
        return this.currentServiceSearch;
    }
    public void setDevices (LinkedList<RemoteDevice> devices) {
        this.devices = devices;
    }
    public void setServiceDevices (Hashtable<Integer, RemoteDevice> serviceDevices) {
        this.serviceDevices = serviceDevices;
    }
    public void addServiceSearch () {
        this.currentServiceSearch++;
    }
    public void deviceDiscovered(RemoteDevice remoteDevice, DeviceClass deviceClass) {
        System.out.println("DEVICE: "+remoteDevice.getBluetoothAddress());
        inquiryDeviceNumber++;
        String fn;
        try {
            fn = remoteDevice.getFriendlyName(true);
        }
        catch (IOException e) {
            System.out.println ("ERROR - El dispositivo no puede ser contactado o no puede proveer su nombre: "+e);
            return;
        }
        String bta = remoteDevice.getBluetoothAddress();
        if (this.database.isEmployeeDevice(bta)) {
            if (this.database.isWorkingEmployee(bta)) {
                return;
            }
        }
        if (this.database.addDevice(fn, bta, this.zone)) {
            this.devices.add(remoteDevice);
        }
        else {
            if (this.database.isNewVisit(bta, this.zone, this.newVisit)) {
                this.database.addVisit(bta, this.zone);
                if (this.database.isServiceRoute(bta)) {
                    System.out.println("Detectado dispositivo a servir: "+remoteDevice.getBluetoothAddress());
                    
                    if (!this.sirviendo.contains(remoteDevice)){                            
                        this.servir.add(remoteDevice);
                        System.out.println("AÑADIDO "+remoteDevice+" a la lista a servir"); 
                    }
                    new BluetoothWorker(this, remoteDevice, this.sirviendo,this.database, this.fileServer, this.zone, this.serveCount);
                    
                }
                else {
                    this.devices.add(remoteDevice);
                }
            }
        }
    }
    public void inquiryCompleted (int completed) {
                
        System.out.println("Inquiry Completed");
        this.inquiryDeviceNumber = 0;
        
        int tam = this.servir.size();
            for ( int i = 0; i < tam; i++) {
                this.serveCount++;
                System.out.println("Tratando dispositivo nº"+(i+1)+" = "+this.servir.peek().getBluetoothAddress());
                new BluetoothWorker(this, this.servir.poll(), this.sirviendo, this.database, this.fileServer, this.zone, i+1);
            }
        
        synchronized (this) {
            this.notify();
        }
    }
    public void servicesDiscovered(int transID, ServiceRecord[] servRecord) {
        for (int i = 0; i < servRecord.length; i++) {
            System.out.println("Encontrado servicio en: "+transID);
            String mac = this.serviceDevices.get(transID).getBluetoothAddress();
            String serviceRoute = servRecord[i].getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT, true);
            this.database.addServiceRoute(mac, serviceRoute);
            System.out.println("Sirviendo dispositivo "+mac);
            new BluetoothWorker (this, this.serviceDevices.get(transID), this.sirviendo, this.database, this.fileServer, this.zone,this.serveCount);
        }
    }
    public void serviceSearchCompleted (int transID, int respCode) {
        System.out.println("Search completed: "+transID);
        this.currentServiceSearch--;
        this.serviceDevices.remove(transID);
        System.out.println("Dispositivos a tratar restantes: "+this.serviceDevices.size());
        if (this.serviceDevices.size() == 0) {
            synchronized (this) {
                this.notify();
            }
        }
    }
}