package accesspoint;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import javax.bluetooth.RemoteDevice;
import javax.microedition.io.Connection;
import javax.obex.ClientSession;
import javax.obex.HeaderSet;
import javax.obex.Operation;
import java.util.Vector;
import java.util.LinkedList;
import de.avetana.bluetooth.connection.Connector;
import database.DatabaseAccess;
public class BluetoothWorker implements Runnable {
    private DatabaseAccess database;
    private RemoteDevice device;
    private FileServerAccess fileServer;
    private boolean stop;
    private Thread thread;
    private int zone;
    
    public int count = 0;
    public BluetoothListener bl;
    public LinkedList<RemoteDevice> servir;
    
    public BluetoothWorker (BluetoothListener bl, RemoteDevice device, LinkedList<RemoteDevice> servir, DatabaseAccess database, FileServerAccess fileServer, int zone, int count) {
        this.database = database;
        this.device = device;
        this.fileServer = fileServer;
        this.stop = false;
        this.thread = new Thread(this);
        thread.start();
        this.zone = zone;
        this.count = count;
        this.bl = bl;
        this.servir = servir;
    }
    public void run () {
        while (!this.stop){
            try {
                this.work();
            }
            catch(Exception e) {
                System.out.println("ERROR - : "+e);
                e.printStackTrace();
                this.stop();
            }
        }
    }
    public void stop () {
        this.stop = true;
    }
    private void work () {
        String serviceRoute = this.database.getServiceRoute(this.device.getBluetoothAddress());
        System.out.println("El Worker esta mandando informacion a "+this.device.getBluetoothAddress());
        Vector<String> items = null;
        int files2send = 1;
     
        
        items = this.database.getSpecificAdList(this.device.getBluetoothAddress(),this.zone);
        if(!items.isEmpty()) {
            for (int i = 0; i < items.size(); i = i+4) {
                if (!this.database.isSpecificAdSent(Integer.parseInt(items.get(i+3)), this.device.getBluetoothAddress())&&files2send != 0) {
                    if (!this.sendFile(serviceRoute, items.get(i+1), "image")) {
                        System.out.println("WARNING - No ha podido enviarse la informacion al dispositivo "+this.device.getBluetoothAddress());
                    }
                    files2send--;
                    this.database.setSpecificAdSent(Integer.parseInt(items.get(i+3)),this.device.getBluetoothAddress());
                }
            }
        }
        else {
            System.out.println("WARNING - No hay articulos que enviar");
        }
     
        this.stop();
    }
    private boolean sendFile (String connectionRoute, String file, String type) {
        
        System.out.println("Buscando el archivo "+file);
        Connection connection = null;
        InputStream isImagen = null;
        try {
            if (!this.fileServer.hasFile(file)) {
                System.out.println("El archivo no se encuentra en el punto de acceso, pidiendo al servidor...");
                this.fileServer.getFile(file, type, 0);
            }
            isImagen = new FileInputStream("./"+type+"/"+file);
        }
        catch (FileNotFoundException e) {
            System.out.println("Fichero no localizado");
        }
        
        try {
            connection = Connector.open(connectionRoute);
        }
        catch (IOException e) {
            System.out.println("ERROR - No se ha abierto la conexion al dispositivo "+this.device.getBluetoothAddress()+" Bluetooth: "+e);
            e.printStackTrace();
            return false;
        }
        ClientSession cs = (ClientSession) connection;
        HeaderSet hs = cs.createHeaderSet();
        try {
            cs.connect(hs);
        }
        catch (Exception e) {
            System.out.println("ERROR - Imposible realizar la conexion: "+e);
            return false;
        }
        Operation putOperation = null;
        OutputStream outputStream = null;
        int bytes = 0;
        try {
            bytes = isImagen.available();
        }
        catch (IOException e) {
            System.out.println("ERROR - La imagen no esta disponible: "+e);
            return false;
        }
        byte filebytesImagen[] = new byte[bytes];
        try {
            isImagen.read(filebytesImagen);
        }
        catch (Exception e) {
            System.out.println("ERROR - La imagen no puede leerse: "+e);
            return false;
        }
        try {
            isImagen.close();
        }
        catch (IOException e) {
            System.out.println("ERROR - La imagen no puede cerrarse: "+e);
            return false;
        }
        hs.setHeader(HeaderSet.NAME, file+""+Math.random());
        if (type.equalsIgnoreCase("image")) {
            hs.setHeader(HeaderSet.TYPE, "image/jpg");
        }
        else {
            hs.setHeader(HeaderSet.TYPE, "application/jar");
        }
        hs.setHeader(HeaderSet.LENGTH, new Long(filebytesImagen.length));
        try {
            putOperation = cs.put(hs);
        }
        catch (IOException e) {
            System.out.println("ERROR - Imposible escribir la cabecera del mensaje: "+e);
            return false;
        }
        try {
            outputStream = putOperation.openOutputStream();
        }
        catch (Exception e) {
            System.out.println("ERROR - El stream de salida no puede abrirse: "+e);
            return false;
        }
        try {
            outputStream.write(filebytesImagen);
            outputStream.flush();
            outputStream.close();
            putOperation.close();
        }
        catch (IOException e) {
            System.out.println("ERROR - Problema con el cerrado de streams: "+e);
            return false;
        }
        try {
            cs.disconnect(null);
        }
        catch (IOException e) {
            System.out.println("ERROR - Imposible desconectar la sesion cliente"+e);
            return false;
        }
        try {
            cs.close();
            connection.close();
        }
        catch (IOException e) {
            System.out.println("ERROR - Imposible cerrar la conexion"+e);
            return false;
        }
        if (this.servir.contains(this.device)){
            this.servir.remove(this.device);
            System.out.println("RETIRADO "+this.device.getBluetoothAddress()+" de la lista a servir");
        }
       
        return true;
    }
}