package fileserver;

import java.io.File;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class FileServer {
    private int port;
    private ServerSocket serverSocket;
    public FileServer (String configurationFile) {
        this.configureFileServer(configurationFile);
        try{
            serverSocket = new ServerSocket(port);
        }
        catch (IOException e) {
            System.out.println("ERROR - No ha podido crearse el socket del servidor: "+e);
        }
    }
    public void serve () {
        while (true) {
            Socket client = null;
            try {
                client = serverSocket.accept();
            }
            catch (IOException e) {
                System.out.println("ERROR - Problemas al aceptar un cliente: "+e);
            }
            if (client != null) {
                System.out.println ("Accepted client");
                new FileServerWorker(client);
            }
        }
    }
    private void configureFileServer (String ConfigurationFile) {
        System.out.print("Setting file server's configuration...");
        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = null;
        Document doc = null;
        try {
            docBuilder = docBuilderFactory.newDocumentBuilder();
        }
        catch (ParserConfigurationException e) {
            System.out.println("ERROR - Imposible crear un nuevo DocumentBuilder: "+e);
            return;
        }
        try {
            doc = docBuilder.parse (new File(ConfigurationFile));
        }
        catch (IOException e) {
            System.out.println("ERROR - Problemas de lectura del fichero de configuracion: "+e);
            return;
        }
        catch (SAXException e) {
            System.out.println("ERROR - Problemas de transformacion del fichero de configuracion: "+e);
            return;
        }
        catch (IllegalArgumentException e) {
            System.out.println("ERROR - Fichero de configuracion no localizado: "+e);
            return;
        }
        doc.getDocumentElement().normalize();
        NodeList portList = doc.getElementsByTagName("port");
        if (portList.getLength() == 1) {
            Element portElement = (Element)portList.item(0);
            NodeList portNumber = portElement.getChildNodes();
            this.port = Integer.parseInt(((Node)portNumber.item(0)).getNodeValue().trim());
        }
        else {
            System.out.println("ERROR - No existe un unico puerto en la configuracion");
            return;
        }
        System.out.println("OK");
        System.out.println("Waiting requests on port "+this.port);
    }
}