package fileserver;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.text.SimpleDateFormat;
import java.util.Date;
import message.RequestFileMessage;
import message.SendFileMessage;

public class FileServerWorker implements Runnable {
    private Socket client;
    private boolean stop;
    private Thread thread;
    public FileServerWorker (Socket client) {
        this.client = client;
        this.stop = false;
        this.thread = new Thread(this);
        thread.start();
    }
    public void run () {
        while (!this.stop){
            this.serveFile();
        }
    }
    public void stop () {
        this.stop = true;
        try {
            this.client.close();
        }
        catch (IOException e) {
            System.out.println ("ERROR - No ha podido cerrarse el socket cliente: "+e);
        }
    }
    private boolean isModifiedFile(File f, long lastModified) {
        if (f != null) {
            if (f.lastModified() > lastModified) {
                return true;
            }
            else {
                return false;
            }
        }
        return true;
    }
    private boolean sendFile(String file, String type, ObjectOutputStream oos) {
        FileInputStream fis;
        int readed = -1;
        SendFileMessage message = new SendFileMessage();
        SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss");
        Date fecha = new Date();
        try {
            fis = new FileInputStream("./"+type+"/" + file);
        }
        catch (FileNotFoundException e) {
            System.out.println ("ERROR - No ha podido abrirse el fichero demandado: "+e);
            return false;
        }
        do {
            message = new SendFileMessage();
            message.name = file;
            try {
                readed = fis.read(message.fileContent);
            }
            catch (IOException e) {
                System.out.println ("ERROR - No ha podido leerse del stream de entrada: "+e);
                return false;
            }
            message.validBytes = readed;
            if (readed < message.maxLong) {
                message.last = true;
            }
            else
                message.last = false;
            if (message.validBytes < 0)
                message.validBytes = 0;
            try {
                oos.writeObject(message);
            }
            catch (IOException e) {
                System.out.println ("ERROR - No ha podido enviarse el mensaje: "+e);
                return false;
            }
        } while (readed == message.maxLong);
        try {
            fis.close();
            oos.close();
        }
        catch (IOException e) {
            System.out.println ("ERROR - No han podido cerrarse los streams: "+e);
            return false;
        }
        fecha = new Date(fecha.getTime());
        System.out.println("File "+file+ " sent succesfully: " + dateFormat.format(fecha));
        return true;
    }
    private void sendNotFoundMessage(String name, ObjectOutputStream oos) {
        SendFileMessage message = new SendFileMessage();
        message.info = "NotFound";
        message.name = name;
        try {
            oos.writeObject(message);
        }
        catch (IOException e) {
            System.out.println ("ERROR - No ha podido enviarse el mensaje: "+e);
            return;
        }
    }
    private void sendNotModifiedMessage(String name, ObjectOutputStream oos) {
        SendFileMessage message = new SendFileMessage();
        message.info = "NotModified";
        message.name = name;
        try {
            oos.writeObject(message);
        }
        catch (IOException e) {
            System.out.println ("ERROR - No ha podido enviarse el mensaje: "+e);
            return;
        }
    }
    private void serveFile () {
        Object message = null;
        ObjectOutputStream oos = null;
        File f = null;
        try {
            ObjectInputStream ois = new ObjectInputStream(this.client.getInputStream());
            message = ois.readObject();
        }
        catch (IOException e) {
            System.out.println("ERROR - No ha podido enlazarse con el stream de entrada del socket cliente: "+e);
            this.stop();
            return;
        }
        catch (ClassNotFoundException e) {
            System.out.println("ERROR - La clase de objeto serializado es desconocida: "+e);
            this.stop();
            return;
        }
        if ((message instanceof RequestFileMessage)&&(message != null)) {
            try {
                oos = new ObjectOutputStream(client.getOutputStream());
            }
            catch (IOException e) {
                System.out.println("ERROR - No ha podido enlazarse con el stream de salida del socket cliente: "+e);
                this.stop();
                return;
            }
            System.out.println("File requested: "+((RequestFileMessage) message).name);
            try {
                f = new File ("./"+((RequestFileMessage)message).type+"/" + ((RequestFileMessage)message).name);
            }catch (NullPointerException e) {
                System.out.println("ERROR - Ruta al fichero erronea: "+e);
                this.stop();
                return;
            }
            try {
                new FileInputStream(f);
            }
            catch (FileNotFoundException e) {
                System.out.println("ERROR - Fichero demandado no existente");
                sendNotFoundMessage(((RequestFileMessage)message).name, oos);
                this.stop();
                return;
            }
            if (!isModifiedFile(f, ((RequestFileMessage)message).lastModified)) {
                System.out.println ("Fichero "+((RequestFileMessage)message).name+" no modificado");
                sendNotModifiedMessage(((RequestFileMessage)message).name, oos);
                this.stop();
                return;
            }
            else {
                if(!sendFile(((RequestFileMessage) message).name, ((RequestFileMessage) message).type, oos)) {
                    System.out.println("ERROR - El fichero "+((RequestFileMessage) message).name+" no se ha enviado correctamente");
                    this.stop();
                    return;
                }
                this.stop();
                return;
            }
        }
        else {
            System.out.println ("WARNING - Recibido mensaje desconocido: "+message.getClass().getName());
        }
    }
}